@@ -264,7 +264,7 @@ generated by running puppet agent with '--genconfig'.
 * --test:
   Enable the most common options used for testing. These are 'onetime',
   'verbose', 'no-daemonize', 'no-usecacheonfailure', 'detailed-exitcodes',
-  'no-splay', and 'show_diff'.
+  'no-splay', 'show_diff', and 'no-use_cached_catalog'.
 
 * --verbose:
   Turn on verbose reporting.
@@ -377,6 +377,7 @@ Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
     Puppet.settings.handlearg("--no-splay")
     Puppet.settings.handlearg("--show_diff")
     Puppet.settings.handlearg("--no-daemonize")
+    Puppet.settings.handlearg("--no-use_cached_catalog")
     options[:verbose] = true
     Puppet[:onetime] = true
     options[:detailed_exitcodes] = true
