@@ -45,5 +45,5 @@ Would notice the value `20`
 * Since 4.5.0
 DOC
 ) do |args|
-  function_fail(["lest() is only available when parser/evaluator future is in effect"])
+  Error.is4x('lest')
 end
