@@ -635,7 +635,7 @@ Defaults to `s` at top level and `p` inside array or hash.
 
 | Format    | Regexp Formats
 | ----      | --------------
-| s         | Delimiters `/ /`, alternate flag `#` replaces `/` delimiters with quotes.
+| s         | No delimiters, quoted if alternative flag `#` is used.
 | p         | Delimiters `/ /`.
 
 ### Undef to String
@@ -775,6 +775,19 @@ Accepts a single value as argument:
 Conversion to a `Struct` works exactly as conversion to a `Hash`, only that the constructed hash is
 asserted against the given struct type.
 
+Conversion to a Regexp
+---
+A `String` can be converted into a `Regexp`
+
+**Example**: Converting a String into a Regexp
+```puppet
+$s = '[a-z]+\.com'
+$r = Regexp($s)
+if('foo.com' =~ $r) {
+  ...
+}
+```
+
 Creating a SemVer
 ---
 
