@@ -51,7 +51,7 @@ class StringConverter
 
   # Format represents one format specification that is textually represented by %<flags><width>.<precision><format>
   # Format parses and makes the individual parts available when an instance is created.
-  # 
+  #
   # @api private
   #
   class Format
@@ -172,7 +172,7 @@ class StringConverter
     # Sorts format based on generality of types - most specific types before general
     #
     def self.sort_formats(format_map)
-      format_map = format_map.sort do |(a,_),(b,_)| 
+      format_map = format_map.sort do |(a,_),(b,_)|
         ab = b.assignable?(a)
         ba = a.assignable?(b)
         if a == b
@@ -219,7 +219,7 @@ class StringConverter
     end
     # Returns an array with a delimiter pair derived from the format.
     # If format does not contain a delimiter specification the given default is returned
-    # 
+    #
     # @param [Array<String>] the default delimiters
     # @returns [Array<String>] a tuple with left, right delimiters
     #
@@ -279,6 +279,7 @@ class StringConverter
     PNumericType::DEFAULT  => Format.new('%d').freeze,    # decimal number
     PArrayType::DEFAULT    => DEFAULT_ARRAY_FORMAT.freeze,
     PHashType::DEFAULT     => DEFAULT_HASH_FORMAT.freeze,
+    PBinaryType::DEFAULT   => Format.new('%B').freeze,    # strict base64 string unquoted
     PAnyType::DEFAULT      => Format.new('%s').freeze,    # unquoted string
   }.freeze
 
@@ -288,7 +289,7 @@ class StringConverter
   # When converting to string it is possible to use a set of built in conversion rules.
   #
   # A format is specified on the form:
-  # 
+  #
   # ´´´
   # %[Flags][Width][.Precision]Format
   # ´´´
@@ -347,9 +348,9 @@ class StringConverter
   # Defaults to `s` at top level and `p` inside array or hash.
   #
   # ### Boolean
-  # 
+  #
   # | Format    | Boolean Formats
-  # | ----      | -------------------   
+  # | ----      | -------------------
   # | t T       | 'true'/'false' or 'True'/'False' , first char if alternate form is used (i.e. 't'/'f' or 'T'/'F').
   # | y Y       | 'yes'/'no', 'Yes'/'No', 'y'/'n' or 'Y'/'N' if alternative flag # is used
   # | dxXobB    | numeric value 0/1 in accordance with the given format which must be valid integer format
@@ -378,13 +379,28 @@ class StringConverter
   # | u         | 'undef', or 'undefined' if alternative # flag is used
   #
   # ### Default (value)
-  # 
+  #
   # | Format    | Default formats
   # | ------    | ---------------
   # | d D       | 'default' or 'Default', alternative form # causes value to be quoted
   # | s         | same as d
   # | p         | same as d
   #
+  # ### Binary (value)
+  #
+  # | Format    | Default formats
+  # | ------    | ---------------
+  # | s         | binary as unquoted characters
+  # | p         | 'Binary("<base64strict>")'
+  # | b         | '<base64>' - base64 string with newlines inserted
+  # | B         | '<base64strict>' - base64 strict string (without newlines inserted)
+  # | u         | '<base64urlsafe>' - base64 urlsafe string
+  # | t         | 'Binary' - outputs the name of the type only
+  # | T         | 'BINARY' - output the name of the type in all caps only
+  #
+  # The alternate form flag `#` will quote the binary or base64 text output
+  # The width and precision values are applied to the text part only in `%p` format.
+  #
   # ### Array & Tuple
   #
   # | Format    | Array/Tuple Formats
@@ -404,18 +420,18 @@ class StringConverter
   #
   # | Format    | Hash/Struct Formats
   # | ------    | -------------
-  # | h         | formats with `{ }` delimiters, `,` element separator and ` => ` inner element separator unless overridden by flags 
+  # | h         | formats with `{ }` delimiters, `,` element separator and ` => ` inner element separator unless overridden by flags
   # | s         | same as h
   # | p         | same as h
   # | a         | converts the hash to an array of [k,v] tuples and formats it using array rule(s)
-  # 
+  #
   # See "Flags" `<[({\|` for formatting of delimiters, and "Additional parameters for containers; Array and Hash" for
   # more information about options.
   #
   # The alternate form flag `#` will format each hash key/value entry indented on a separate line.
   #
   # ### Type
-  # 
+  #
   # | Format    | Array/Tuple Formats
   # | ------    | -------------
   # | s        | The same as p, quoted if alternative flag # is used
@@ -423,7 +439,7 @@ class StringConverter
   #
   # ### Flags
   #
-  # | Flag     | Effect 
+  # | Flag     | Effect
   # | ------   | ------
   # | (space)  | space instead of + for numeric output (- is shown), for containers skips delimiters
   # | #        | alternate format; prefix 0x/0x, 0 (octal) and 0b/0B for binary, Floats force decimal '.'. For g/G keep trailing 0.
@@ -432,7 +448,7 @@ class StringConverter
   # | 0        | pad with 0 instead of space for widths larger than value
   # | <[({\|   | defines an enclosing pair <> [] () {} or \| \| when used with a container type
   #
-  # 
+  #
   # ### Additional parameters for containers; Array and Hash
   #
   # For containers (Array and Hash), the format is specified by a hash where the following keys can be set:
@@ -440,7 +456,7 @@ class StringConverter
   # * `'separator'` - the separator string to use between elements, should not contain padding space at the end
   # * `'separator2'` - the separator string to use between association of hash entries key/value
   # * `'string_formats'´ - a map of type to format for elements contained in the container
-  # 
+  #
   # Note that the top level format applies to Array and Hash objects contained/nested in an Array or a Hash.
   #
   # Given format mappings are merged with (default) formats and a format specified for a narrower type
@@ -522,8 +538,9 @@ class StringConverter
   end
   private :validate_container_input
 
-  def string_PRuntimeType(val_type, val, format_map)
-    case (f = get_format(val_type, format_map))
+  def string_PRuntimeType(val_type, val, format_map, _)
+    f = get_format(val_type, format_map)
+    case f.format
     when :s
       val.to_s
     when :q
@@ -534,7 +551,7 @@ class StringConverter
       converted = convert(o, PNumericType) # rest is default
       "%#{f}" % converted
     else
-      throw(:failed_conversion, [o, PStringType::DEFAULT, f])
+      raise FormatError.new('Runtime', f.format, 'sqidxof')
     end
   end
 
@@ -696,6 +713,52 @@ class StringConverter
     end
   end
 
+  # @api private
+  def string_PBinaryType(val_type, val, format_map, _)
+    f = get_format(val_type, format_map)
+    substitute = f.alt? ? 'p' : 's'
+    case f.format
+    when :s
+      val_to_convert = val.binary_buffer
+      if !f.alt?
+        # Assume it is valid UTF-8
+        val_to_convert = val_to_convert.dup.force_encoding('UTF-8')
+        # If it isn't
+        unless val_to_convert.valid_encoding?
+          # try to convert and fail with details about what is wrong
+          val_to_convert = val.binary_buffer.encode('UTF-8')
+        end
+      else
+        val_to_convert = val.binary_buffer
+      end
+      Kernel.format(f.orig_fmt.gsub('s', substitute), val_to_convert)
+
+    when :p
+      # width & precision applied to string, not the the name of the type
+      "Binary(\"#{Kernel.format(f.orig_fmt.gsub('p', 's'), val.to_s)}\")"
+
+    when :b
+      Kernel.format(f.orig_fmt.gsub('b', substitute), val.relaxed_to_s)
+
+    when :B
+      Kernel.format(f.orig_fmt.gsub('B', substitute), val.to_s)
+
+    when :u
+      Kernel.format(f.orig_fmt.gsub('u', substitute), val.urlsafe_to_s)
+
+    when :t
+      # Output as the type without any data
+      Kernel.format(f.orig_fmt.gsub('t', substitute), 'Binary')
+
+    when :T
+      # Output as the type without any data in all caps
+      Kernel.format(f.orig_fmt.gsub('T', substitute), 'BINARY')
+
+    else
+      raise FormatError.new('Binary', f.format, 'bButTsp')
+    end
+  end
+
   # @api private
   def string_PStringType(val_type, val, format_map, _)
     f = get_format(val_type, format_map)
@@ -776,13 +839,15 @@ class StringConverter
     f = get_format(val_type, format_map)
     case f.format
     when :p
-      Kernel.format(f.orig_fmt, val)
+      str_regexp = '/'
+      str_regexp << (val.options == 0 ? val.source : val.to_s) << '/'
+      f.orig_fmt == '%p' ? str_regexp : Kernel.format(f.orig_fmt.gsub('p', 's'), str_regexp)
     when :s
-      str_regexp = val.inspect
-      str_regexp = f.alt? ? "\"#{str_regexp[1..-2]}\"" : str_regexp
-      Kernel.format(f.orig_fmt, str_regexp)
+      str_regexp = val.options == 0 ? val.source : val.to_s
+      str_regexp = puppet_quote(str_regexp) if f.alt?
+      f.orig_fmt == '%s' ? str_regexp : Kernel.format(f.orig_fmt, str_regexp)
     else
-      raise FormatError.new('Regexp', f.format, 'rsp')
+      raise FormatError.new('Regexp', f.format, 'sp')
     end
   end
 
