@@ -312,7 +312,7 @@ class TypeCalculator
         infer_set_Array(o)
       when Hash
         infer_set_Hash(o)
-      when Semantic::Version
+      when SemanticPuppet::Version
         infer_set_Version(o)
       else
         infer_set_Object(o)
@@ -389,8 +389,8 @@ class TypeCalculator
     # when both are hashes, return a hash with common key- and element type
     if t1.is_a?(PHashType) && t2.is_a?(PHashType)
       key_type = common_type(t1.key_type, t2.key_type)
-      element_type = common_type(t1.element_type, t2.element_type)
-      return PHashType.new(key_type, element_type)
+      value_type = common_type(t1.value_type, t2.value_type)
+      return PHashType.new(key_type, value_type)
     end
 
     # when both are host-classes, reduce to PHostClass[] (since one was not assignable to the other)
@@ -414,19 +414,29 @@ class TypeCalculator
       return PFloatType.new([t1.numeric_from, t2.numeric_from].min, [t1.numeric_to, t2.numeric_to].max)
     end
 
-    if t1.is_a?(PStringType) && t2.is_a?(PStringType)
-      common_size_type = common_type(t1.size_type, t2.size_type) unless t1.size_type.nil? || t2.size_type.nil?
-      common_strings = t1.values.empty? || t2.values.empty? ? [] : t1.values | t2.values
-      return PStringType.new(common_size_type, common_strings)
+    if t1.is_a?(PStringType) && (t2.is_a?(PStringType) || t2.is_a?(PEnumType))
+      if(t2.is_a?(PEnumType))
+        return t1.value.nil? ? PEnumType::DEFAULT : PEnumType.new(t2.values | [t1.value])
+      end
+
+      if t1.size_type.nil? || t2.size_type.nil?
+        return t1.value.nil? || t2.value.nil? ? PStringType::DEFAULT : PEnumType.new([t1.value, t2.value])
+      end
+
+      return PStringType.new(common_type(t1.size_type, t2.size_type))
     end
 
     if t1.is_a?(PPatternType) && t2.is_a?(PPatternType)
       return PPatternType.new(t1.patterns | t2.patterns)
     end
 
-    if t1.is_a?(PEnumType) && t2.is_a?(PEnumType)
+    if t1.is_a?(PEnumType) && (t2.is_a?(PStringType) || t2.is_a?(PEnumType))
       # The common type is one that complies with either set
-      return PEnumType.new(t1.values | t2.values)
+      if t2.is_a?(PEnumType)
+        return PEnumType.new(t1.values | t2.values)
+      end
+
+      return t2.value.nil? ? PEnumType::DEFAULT : PEnumType.new(t1.values | [t2.value])
     end
 
     if t1.is_a?(PVariantType) && t2.is_a?(PVariantType)
@@ -437,7 +447,7 @@ class TypeCalculator
     if t1.is_a?(PRegexpType) && t2.is_a?(PRegexpType)
       # if they were identical, the general rule would return a parameterized regexp
       # since they were not, the result is a generic regexp type
-      return PPatternType::DEFAULT
+      return PRegexpType::DEFAULT
     end
 
     if t1.is_a?(PCallableType) && t2.is_a?(PCallableType)
@@ -581,7 +591,7 @@ class TypeCalculator
 
   # @api private
   def infer_String(o)
-    PStringType.new(size_as_type(o), [o])
+    PStringType.new(o)
   end
 
   # @api private
@@ -620,9 +630,8 @@ class TypeCalculator
       max += 1
       PAnyType::DEFAULT
     end
-    mapped_types << min
-    mapped_types << max
-    TypeFactory.callable(*mapped_types)
+    param_types = Types::PTupleType.new(mapped_types, Types::PIntegerType.new(min, max))
+    Types::PCallableType.new(param_types)
   end
 
   # @api private
@@ -648,6 +657,16 @@ class TypeCalculator
     PSensitiveType.new(infer(o.unwrap))
   end
 
+  # @api private
+  def infer_Timespan(o)
+    PTimespanType.new(o, o)
+  end
+
+  # @api private
+  def infer_Timestamp(o)
+    PTimestampType.new(o, o)
+  end
+
   # @api private
   def infer_TrueClass(o)
     PBooleanType::DEFAULT
@@ -678,6 +697,11 @@ class TypeCalculator
     end
   end
 
+  # @api private
+  def infer_Binary(o)
+    PBinaryType::DEFAULT
+  end
+
   # @api private
   def infer_Version(o)
     PSemVerType::DEFAULT
@@ -721,7 +745,7 @@ class TypeCalculator
     if o.empty?
       PHashType::EMPTY
     elsif o.keys.all? {|k| PStringType::NON_EMPTY.instance?(k) }
-      PStructType.new(o.each_pair.map { |k,v| PStructElement.new(PStringType.new(size_as_type(k), [k]), infer_set(v)) })
+      PStructType.new(o.each_pair.map { |k,v| PStructElement.new(PStringType.new(k), infer_set(v)) })
     else
       ktype = PVariantType.maybe_create(o.keys.map {|k| infer_set(k) })
       etype = PVariantType.maybe_create(o.values.map {|e| infer_set(e) })
@@ -731,7 +755,7 @@ class TypeCalculator
 
   # @api private
   def infer_set_Version(o)
-    PSemVerType.new([Semantic::VersionRange.new(o, o)])
+    PSemVerType.new([SemanticPuppet::VersionRange.new(o, o)])
   end
 
   def unwrap_single_variant(possible_variant)
