@@ -101,7 +101,7 @@ class OpenSSL::SSL::SSLContext
   else
     DEFAULT_PARAMS[:options] = OpenSSL::SSL::OP_NO_SSLv2 | OpenSSL::SSL::OP_NO_SSLv3
   end
-  DEFAULT_PARAMS[:ciphers] << ':!SSLv2'
+#  DEFAULT_PARAMS[:ciphers] << ':!SSLv2'
 
   alias __original_initialize initialize
   private :__original_initialize
